@@ -1,6 +1,6 @@
 package Net::Stomp::Producer::Exceptions;
 {
-  $Net::Stomp::Producer::Exceptions::VERSION = '1.0';
+  $Net::Stomp::Producer::Exceptions::VERSION = '1.3.1';
 }
 {
   $Net::Stomp::Producer::Exceptions::DIST = 'Net-Stomp-Producer';
@@ -13,7 +13,7 @@ use Net::Stomp::MooseHelpers::Exceptions;
 {
 package Net::Stomp::Producer::Exceptions::StackTrace;
 {
-  $Net::Stomp::Producer::Exceptions::StackTrace::VERSION = '1.0';
+  $Net::Stomp::Producer::Exceptions::StackTrace::VERSION = '1.3.1';
 }
 {
   $Net::Stomp::Producer::Exceptions::StackTrace::DIST = 'Net-Stomp-Producer';
@@ -40,7 +40,7 @@ around _build_stack_trace_args => sub {
 {
 package Net::Stomp::Producer::Exceptions::BadMessage;
 {
-  $Net::Stomp::Producer::Exceptions::BadMessage::VERSION = '1.0';
+  $Net::Stomp::Producer::Exceptions::BadMessage::VERSION = '1.3.1';
 }
 {
   $Net::Stomp::Producer::Exceptions::BadMessage::DIST = 'Net-Stomp-Producer';
@@ -53,6 +53,7 @@ use Data::Dump 'pp';
 has message_body => ( is => 'ro', required => 1 );
 has message_headers => ( is => 'ro', required => 0 );
 has reason => ( is => 'ro', default => q{sending the message didn't work} );
+has '+previous_exception' => ( init_arg => 'previous_exception' );
 
 sub as_string {
     my ($self) = @_;
@@ -67,7 +68,7 @@ __PACKAGE__->meta->make_immutable(inline_constructor=>0);
 {
 package Net::Stomp::Producer::Exceptions::CantSerialize;
 {
-  $Net::Stomp::Producer::Exceptions::CantSerialize::VERSION = '1.0';
+  $Net::Stomp::Producer::Exceptions::CantSerialize::VERSION = '1.3.1';
 }
 {
   $Net::Stomp::Producer::Exceptions::CantSerialize::DIST = 'Net-Stomp-Producer';
@@ -80,7 +81,7 @@ __PACKAGE__->meta->make_immutable(inline_constructor=>0);
 {
 package Net::Stomp::Producer::Exceptions::BadTransformer;
 {
-  $Net::Stomp::Producer::Exceptions::BadTransformer::VERSION = '1.0';
+  $Net::Stomp::Producer::Exceptions::BadTransformer::VERSION = '1.3.1';
 }
 {
   $Net::Stomp::Producer::Exceptions::BadTransformer::DIST = 'Net-Stomp-Producer';
@@ -102,7 +103,7 @@ __PACKAGE__->meta->make_immutable(inline_constructor=>0);
 {
 package Net::Stomp::Producer::Exceptions::Invalid;
 {
-  $Net::Stomp::Producer::Exceptions::Invalid::VERSION = '1.0';
+  $Net::Stomp::Producer::Exceptions::Invalid::VERSION = '1.3.1';
 }
 {
   $Net::Stomp::Producer::Exceptions::Invalid::DIST = 'Net-Stomp-Producer';
@@ -135,7 +136,7 @@ Net::Stomp::Producer::Exceptions - exception classes for Net::Stomp::Producer
 
 =head1 VERSION
 
-version 1.0
+version 1.3.1
 
 =head1 DESCRIPTION
 
